@@ -12,10 +12,11 @@ from xml.sax._exceptions import SAXParseException
 from rdflib.namespace import split_uri
 from rdflib import compare
 
+import helpers
 import utils
 
-# set socket timeout. URLError will occur
-socket.setdefaulttimeout(20)
+# set socket timeout. URLError will occur if time passed
+socket.setdefaulttimeout(30)
 
 logger = logging.getLogger(__name__)
 
@@ -51,8 +52,10 @@ def predicate2pyattr(predicate, namespace_short_notation_reverse_dict):
     prefix, propertyname = split_uri(predicate)
     assert prefix
     assert propertyname
+
     #if not "_" in propertyname:
     #    logger.info("%s_%s may cause problems?" % (prefix, propertyname))
+
     if not prefix in namespace_short_notation_reverse_dict:
         logger.warning("%s cannot be shortened" % predicate)
         return predicate
@@ -70,7 +73,7 @@ def pyattr2predicate(pyattr, namespace_dict):
 
     splitlist = pyattr.split("_")
 
-    # TODO: check for namespace prefix limitations
+    # this code ckecks pyattr for namespace prefix limitations
     splitlistlen = len(splitlist)
     if splitlistlen == 1:
         # attribute "homepage" --> check if "" in namespace_dict
@@ -117,20 +120,6 @@ class ResourceManager(Manager):
         assert origin.processed, ("Origin has to be processed "
             "before creating more Resource objects: origin.GET()")
 
-        # TODO write testcase for this first
-        ## Absolutize relative URIs
-        #if uri.startswith("#"):
-        #    assert not isinstance(uri, rdflib.BNode), \
-        #        "BNode not allowed to start with #"
-        #    uri = rdflib.URIRef(origin.uri + uri)
-
-        if isinstance(uri, (rdflib.BNode, rdflib.URIRef)):
-            pass
-        elif not utils.is_valid_url(uri):
-            msg = "URI %s not a valid URL" % uri
-            logger.error(msg)
-            raise ValueError(msg)
-
         uri = utils.get_rdflib_uriref(uri)
 
         pk = self.get_pk(origin_uri=origin.uri, uri=uri)
@@ -139,14 +128,10 @@ class ResourceManager(Manager):
 
     def get_authoritative_resource(self, uri,
                                    create_nonexistent_origin=True):
-        """Tries to return the Resource object from the original origin"""
+        """Tries to return the Resource object from the authoritative origin"""
 
-        if not utils.is_valid_url(uri):
-            logger.error("Not a valid Uri: %s" % uri)
-            return
         uri = utils.get_rdflib_uriref(uri)
-
-        origin_uri = utils.hash_to_slash_uri(uri)
+        origin_uri = utils.get_slash_url(uri)
 
         authoritative_origin = Origin.objects.filter(uri=origin_uri)
         authoritative_origin_list = list(authoritative_origin)
@@ -159,7 +144,6 @@ class ResourceManager(Manager):
                 raise self.model.DoesNotExist("No authoritative "
                 "Resource found for %s" %uri)
 
-        # TODO: make this configurable
         if not origin.has_unsaved_changes():
             origin.GET(only_follow_uris=[], raise_errors=False)
 
@@ -202,7 +186,6 @@ class ResourceManager(Manager):
         assert 0, "implement!"
 
     def get_or_create(self, uri, origin=None):
-
         uri = utils.get_rdflib_uriref(uri)
         try:
             return self.get(uri=uri, origin=origin), False
@@ -233,14 +216,14 @@ class Resource(Model):
         if isinstance(self._uri, rdflib.BNode):
             return True
 
-        if utils.hash_to_slash_uri(self._uri) == self._origin.uri:
+        if utils.get_slash_url(self._uri) == self._origin.uri:
             return True
 
     def _add_property(self, predicate, object,
                       namespace_short_notation_reverse_dict):
 
         assert isinstance(predicate, rdflib.URIRef),\
-            "Not an URIRef: %s"%predicate
+            "Not an URIRef: %s" % predicate
         assert hasattr(predicate, "n3"),\
             "property %s is not a rdflib object" % predicate
 
@@ -271,7 +254,8 @@ class Resource(Model):
                 object._reverse = {}
             if predicate in object._reverse:
                 if not isinstance(object._reverse[predicate], set):
-                    object._reverse[predicate] = set([object._reverse[predicate]])
+                    object._reverse[predicate] = set(
+                        [object._reverse[predicate]])
                 object._reverse[predicate].add(self)
             else:
                 object._reverse[predicate] = self
@@ -286,59 +270,9 @@ class Resource(Model):
         else:
             setattr(self, predicate, object)
             attr = getattr(self, predicate)
-            #assert attr == object
 
-        # TODO: this might cause UnicodeDecodeErrors
-        #logger.debug(u"Contribute_to_object %s: %s = %s"
-        #    % (self, predicate.encode('utf8'), attr))
-        
-        # TODO: introduce rdflib.Literal dict or force_unicode when
-        # reconverting from __dict__?
         assert str(predicate) in self.__dict__
 
-    def _tripleserialize_iterator(self,
-                                  namespace_dict):
-
-        def do_yield(resource, property, v):
-            if isinstance(v, resource.__class__):
-                # If object is referenced in attribute, "de-reference"
-                return ((resource._uri, property, v._uri))
-            else:
-
-                if not hasattr(v, "n3"):
-                    # print only newly added values without the correct
-                    # type are handled here
-
-                    if (hasattr(v, "startswith") # float has no attribute startswith
-                        and v.startswith("http://") # TODO: this is not very accurate!
-                        ):
-                        v = rdflib.URIRef(v)
-                    else:
-                        v = rdflib.Literal(v)
-
-                return ((self._uri, property, v))
-
-        for property, values in self.__dict__.items():
-
-            # skip internals
-            if str(property).startswith("_") or property == "pk":
-                continue
-
-            if property.startswith("http://"):
-                property = rdflib.URIRef(property)
-            else:
-                property = pyattr2predicate(property, namespace_dict)
-
-            assert isinstance(property, rdflib.URIRef), \
-                "property %s is not a URIRef object" % property
-
-            if isinstance(values, set):
-                for v in values:
-                    yield do_yield(self, property, v)
-            else:
-                v = values
-                yield do_yield(self, property, v)
-
     def __setattr__(self, key, value):
         if key == "_has_changes":
             Model.__setattr__(self, key, value)
@@ -350,7 +284,8 @@ class Resource(Model):
                 value = field.to_python(value)
         elif not key.startswith("_") and not key == "pk":
             # Assumption: rdf attributes do not start with "_"
-            pass # TODO: validate
+            pass
+
         Model.__setattr__(self, key, value)
         self._has_changes = True
 
@@ -359,13 +294,12 @@ class Resource(Model):
             self.__class__.objects._storage.__delitem__(self.pk)
 
     def save(self):
-        # TODO Move to models.py and introduce "clean" to validate properties
         created = not self.pk
         if created:
             assert 0, ("Please use Resource.objects.create() to create "
                        "Resource objects!")
         assert self in Resource.objects.filter(_origin=self._origin)
-        #values = dict((name, getattr(self, name)) for name in
+        #defined_values = dict((name, getattr(self, name)) for name in
         # self._meta.fields.iterkeys())
         self.update() #**values)
 
@@ -382,9 +316,13 @@ class OriginManager(Manager):
         return origin
 
     def create(self, uri, BACKEND=None):
-        uri = utils.get_rdflib_uriref(uri)
 
-        assert not '#' in uri, ("HashURI not allowed as Origin: %s" % uri)
+        uri = utils.get_rdflib_uriref(uri)
+        if not uri == utils.get_slash_url(uri):
+            msg = ("URI passed to Origin Manager was not a slash URI: %s. "
+                   "Fixed now." % uri)
+            logger.debug(msg)
+            uri = utils.get_slash_url(uri)
 
         backend = BACKEND if BACKEND else RestBackend()
         origin = super(OriginManager, self).create(pk=uri, uri=uri,
@@ -393,26 +331,21 @@ class OriginManager(Manager):
 
     def get(self, uri, **kwargs):
         """Retrieves Origin object from Store"""
-
         uri = utils.get_rdflib_uriref(uri)
-
         return super(OriginManager, self).get(pk=uri)
 
-    def get_or_create(self, uri, fail_silently=True,
-                      **kwargs #TODO: remove kwargs for better api
-                      ):
+    def get_or_create(self, uri,
+                      **kwargs):
 
         uri = utils.get_rdflib_uriref(uri)
-        if not uri == utils.hash_to_slash_uri(uri):
-            msg = "URI is not a slash URI: %s" % uri
-            if fail_silently:
-                logger.error(msg)
-                return None, None
-            else: raise Exception(msg)
+        if not uri == utils.get_slash_url(uri):
+            msg = ("URI passed to Origin Manager was not a slash URI: %s. "
+                   "Fixed now." % uri)
+            logger.warning(msg)
+            uri = utils.get_slash_url(uri)
 
         try:
-            if kwargs:
-                logger.warning("kwargs are ignored for get.")
+            if kwargs: logger.warning("kwargs are ignored for get.")
             return self.get(uri), False
         except self.model.DoesNotExist:
             return self.create(uri, **kwargs), True
@@ -420,9 +353,6 @@ class OriginManager(Manager):
     @catchKeyboardInterrupt
     def GET_all(self, depth=2, **kwargs):
         """Crawls or Re-Crawls all Origins. Passes Arguments to GET"""
-
-        # TODO: limit crawling speed
-
         func = lambda origin: True if not origin.processed else False
         for _i in range(depth):
             crawl = filter(func, self.all())
@@ -430,6 +360,7 @@ class OriginManager(Manager):
                 for origin in crawl:
                     origin.GET(raise_errors=False, **kwargs)
 
+
 class Origin(Model):
 
     uri = URIRefField()
@@ -442,7 +373,6 @@ class Origin(Model):
 
     def __init__(self, pk=None, **kwargs):
         super(Origin, self).__init__(pk=pk, **kwargs)
-        self.stats = {}
         self.processed = False
 
     def __unicode__(self):
@@ -459,14 +389,15 @@ class Origin(Model):
             GRAPH_SIZE_LIMIT=30000,
             only_follow_uris=None,
             handle_owl_imports=False,
-            skip_urls=None,
             raise_errors=True,
+            skip_urls=None,
+            httphandler=None,
             ):
 
         if not self.uri:
             raise Exception("Please provide URI first")
 
-        if skip_urls is not None and str(self.uri) in skip_urls:
+        if skip_urls is not None and self.uri.encode("utf8") in skip_urls:
             self.add_error("Skipped")
             self.processed = True
             return
@@ -483,16 +414,18 @@ class Origin(Model):
 
         now = datetime.datetime.now()
         #self.timedelta = datetime.timedelta(minutes=1)
-        if hasattr(self, "timedelta") and 'last_processed' in self.stats:
-            time_since_last_processed = now - self.stats['last_processed']
+        if hasattr(self, "timedelta") and hasattr(self, 'last_processed'):
+            time_since_last_processed = now - self.last_processed
             if (time_since_last_processed < self.timedelta):
                 logger.info("Not processing %s again because was processed "
                 "only %s ago" % (self.uri, time_since_last_processed))
                 return
-        self.stats['last_processed'] = now
+
+        self.last_processed = now
 
         try:
-            data = self.backend.GET(self.uri)
+            data = self.backend.GET(self.uri,
+                                    httphandler=httphandler)
         except urllib2.HTTPError as e:
             if e.code in [
                     401,
@@ -512,24 +445,25 @@ class Origin(Model):
             if raise_errors: raise e
             else: return
 
-
         graph = rdflib.graph.ConjunctiveGraph(identifier=self.uri)
 
-
         try:
             if data:
                 # Important: Do not pass data=data without publicID=uri because
                 # relative URIs (#deri) won't be an absolute uri in that case!
                 publicID = self.uri
+
+                reference_time = datetime.datetime.now()
+
                 graph.parse(data=data,
                             publicID=publicID,
                             format=self.backend.format)
 
+                now = datetime.datetime.now()
+                self.graph_parse_time = now - reference_time
+
                 # normal rdflib.compare does not work correctly with
                 # ConjunctiveGraph, unless there is only one graph within that
-                if not self.uri in [rdflib.URIRef("http://dbpedia.org/resource/Spinner_(Blade_Runner)")]:
-                    assert len(list(graph.contexts())) == 1
-
         except SAXParseException as e:
             self.add_error("SAXParseException")
             logger.error("SAXParseException: %s" % self)
@@ -541,7 +475,6 @@ class Origin(Model):
             if raise_errors: raise e
             else: return
         except IOError as e:
-            # TODO: why does this occur? guess: wrong protocol
             self.add_error("IOError")
             logger.error("IOError: %s" % self)
             if raise_errors: raise e
@@ -553,6 +486,10 @@ class Origin(Model):
             delattr(self, "errors")
 
         g_length = len(graph)
+
+        if g_length > 0:
+            assert len(list(graph.contexts())) == 1
+
         if g_length > GRAPH_SIZE_LIMIT:
             logger.error("Maximum graph size exceeded. Thr graph is %s "
                          "triples big. Limit is set to %s. The aquired "
@@ -571,10 +508,7 @@ class Origin(Model):
                 return
             else:
                 logging.warning("GET retrieved updates for %s!" % self.uri)
-                try:
-                    utils.my_graph_diff(self._graph, graph)
-                except ImportError:
-                    pass
+                helpers.my_graph_diff(self._graph, graph)
 
                 for resource in self.get_resources():
                     resource.delete()
@@ -585,21 +519,18 @@ class Origin(Model):
 
         self._graph = graph
 
-        start_time = datetime.datetime.now()
-
         graph_handler = GraphHandler(
             only_follow_uris=only_follow_uris,
             handle_owl_imports=handle_owl_imports,
             origin=self)
         graph_handler.populate_resources(graph=graph)
 
-        self.stats['graph_processing_time'] = datetime.datetime.now() - start_time
-
         self.handled = True
 
     def get_graph(self):
         """Processes every Resource and Property related to 'self'"""
-        #rdflib.ConjunctiveGraph because rdflib.Graph does not allow parsing plugins
+        #rdflib.ConjunctiveGraph because rdflib.Graph does not allow
+        # usage of parsing plugins
         graph = rdflib.graph.ConjunctiveGraph(identifier=self.uri)
 
         if not hasattr(self, '_graph'):
@@ -608,12 +539,10 @@ class Origin(Model):
                               ".get_graph()"
                     % self.uri)
                 return graph
+            assert hasattr(self, "_graph"), ("graph has to be "
+                "processed before executing get_graph()")
 
-            #self.GET(raise_errors=False) # TODO: test for recursion?
-
-
-        # TODO: find a better way to do this
-        # Problems:
+        # Problems with namespacemapping here:
         #  1) namespace bindings are not really necessary to validate
         #     isomorphic graphs but the resulting graph is is different
         #     if they miss
@@ -628,13 +557,54 @@ class Origin(Model):
             for k, v in safe_dict(namespace_dict).items() \
                 if not k in safe_dict(new_ns).keys()]
 
+
+        def do_yield_triple(resource, property, v):
+            if isinstance(v, resource.__class__):
+                # If object is referenced in attribute, "de-reference"
+                return ((resource._uri, property, v._uri))
+            else:
+
+                if not hasattr(v, "n3"):
+                    # print only newly added values without the correct
+                    # type are handled here
+
+                    # float has no attribute startswith
+                    if (hasattr(v, "startswith")
+                        and v.startswith("http://")
+                        ):
+                        v = rdflib.URIRef(v)
+                    else:
+                        v = rdflib.Literal(v)
+
+                return ((resource._uri, property, v))
+
+
         for resource in self.get_resources():
-            # TODO: better idea how to do this?
-            # __dict__ converts rdflib.urirefs to strings -->
-            # converts back to uriref
+            # __dict__ converts rdflib.urirefs to strings for keys -->
+            # convert back the dict's items back to uriref
             # {'foaf': 'http:/....', ...}
-            for triple in resource._tripleserialize_iterator(namespace_dict):
-                graph.add(triple)
+
+            for property, values in resource.__dict__.items():
+
+                # skip internals
+                if str(property).startswith("_") or property == "pk":
+                    continue
+
+                if property.startswith("http://"):
+                    property = rdflib.URIRef(property)
+                else:
+                    property = pyattr2predicate(property, namespace_dict)
+
+                assert isinstance(property, rdflib.URIRef), \
+                    "property %s is not a URIRef object" % property
+
+                if isinstance(values, set):
+                    for v in values:
+                        graph.add( do_yield_triple(resource, property, v) )
+                else:
+                    v = values
+                    graph.add( do_yield_triple(resource, property, v) )
+
         return graph
 
     def get_resources(self):
@@ -662,14 +632,12 @@ class Origin(Model):
         graph = self.get_graph()
         data = graph.serialize(format=self.backend.format)
 
-        # TODO: synchronize if remote resource is still up to date?
         self.backend.PUT(data=data)
 
         for resource in Resource.objects.filter(_has_changes=True):
             resource._has_changes = False
 
         assert not self.has_unsaved_changes(), "something went wrong"
-        # TODO return "OK"
 
 
 def check_shortcut_consistency():
@@ -699,18 +667,11 @@ class GraphHandler(object):
         namespace_short_notation_reverse_dict = reverse_dict(dict(graph\
             .namespace_manager.namespaces()))
 
+        reference_time = datetime.datetime.now()
+
         for subject, predicate, obj_ect in graph:
             assert hasattr(subject, "n3")
 
-            #subject = utils.get_rdflib_uriref(subject)
-
-            #if isinstance(subject, rdflib.BNode):
-            #    assert 0, "subject is BNode: %s %s %s" % (subject, predicate, obj_ect)
-            #if isinstance(predicate, rdflib.BNode):
-            #    assert 0, "predicate is BNode: %s %s %s" % (subject, predicate, obj_ect)
-            #if isinstance(obj_ect, rdflib.BNode):
-            #    assert 0, "object is BNode: %s %s %s" % (subject, predicate, obj_ect)
-
             # workaround for rdflib's unicode problems
             assert predicate.encode('utf8')
 
@@ -718,8 +679,7 @@ class GraphHandler(object):
                 if (predicate == rdflib.OWL.imports
                     and type(obj_ect) == rdflib.URIRef):
 
-                    uri = utils.hash_to_slash_uri(obj_ect)
-
+                    uri = utils.get_slash_url(obj_ect)
                     origin, created = Origin.objects.get_or_create(uri=uri)
 
                     logger.info("Interrupting to process owl:imports %s"
@@ -731,8 +691,9 @@ class GraphHandler(object):
                 or self.only_follow_uris is None):
 
                 if type(obj_ect) == rdflib.URIRef:
+                    # wrong scheme mailto, tel, callto --> should be Literal?
                     if utils.is_valid_url(obj_ect):
-                        obj_uriref = utils.hash_to_slash_uri(obj_ect, logger=logger)
+                        obj_uriref = utils.get_slash_url(obj_ect)
                         Origin.objects.get_or_create(uri=obj_uriref)
 
             resource, _created = Resource.objects.get_or_create(uri=subject,
@@ -740,5 +701,8 @@ class GraphHandler(object):
             resource._add_property(predicate, obj_ect,
                                    namespace_short_notation_reverse_dict)
 
+        now = datetime.datetime.now()
+        self.origin.graph_handler_time = now - reference_time
+
         for resource in self.origin.get_resources():
             resource._has_changes = False
