@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-__version__ = "0.5.4"
+__version__ = "0.5.5"
 __useragent__ = ('ldtools-%s (http://github.com/dmr/ldtools, daniel@nwebs.de)'
                  % __version__)
 
@@ -12,14 +12,16 @@ import shutil
 import urllib2
 import glob
 
-
-# add n3 to known mimetypes
+# add mimetypes python does not know yet
 mimetypes.add_type("text/n3", ".n3")
 mimetypes.add_type("text/rdf+n3", ".n3")
+mimetypes.add_type("text/turtle", ".n3")
 
 
-class FiletypeMappingError(Exception): pass
-class ContentNegotiationError(Exception): pass
+class FiletypeMappingError(Exception):
+    pass
+class ContentNegotiationError(Exception):
+    pass
 
 
 logger = logging.getLogger("ldtools")
@@ -31,13 +33,11 @@ def get_file_extension(filename):
 
 
 def assure_parser_plugin_exists(format):
-    # TODO: double check if parser for format exists -->
-    # TODO: move parser to backend
     try:
         rdflib.graph.plugin.get(name=format,
                             kind=rdflib.parser.Parser)
     except rdflib.plugin.PluginException as e:
-        msg = "Not parser plugin found for %s" % format
+        msg = "No parser plugin found for %s" % format
         logger.error(msg)
         raise ContentNegotiationError(msg)
 
@@ -50,9 +50,7 @@ def guess_format_from_filename(file_name):
 
 class AbstractBackend(object):
     """Abstract Backend. Overwrite in subclasses"""
-    def GET(self, uri): raise NotImplementedError
-    def PUT(self): raise NotImplementedError
-
+    pass
 
 class RestBackend(AbstractBackend):
     GET_headers = {
@@ -61,6 +59,7 @@ class RestBackend(AbstractBackend):
             'text/n3,'
             'text/rdf+n3,'
             'application/rdf+xml;q=0.8'
+            "text/turtle;q=0.7,"
             #'application/xhtml+xml;q=0.5'
             #'*/*;q=0.1'
             #XHTML+RDFa
@@ -71,10 +70,12 @@ class RestBackend(AbstractBackend):
         "User-Agent": __useragent__,
     }
 
-    def GET(self, uri, extra_headers=None, debug=False):
-        """lookup URI"""
-        # TODO: do friendly crawling: use robots.txt speed
-        # limitation definitions
+    def GET(self,
+            uri,
+            extra_headers=None,
+            httphandler=None,
+        ):
+        """Lookup URI and follow redirects. Return data"""
 
         if not hasattr(self, "uri"):
             self.uri = uri
@@ -83,34 +84,26 @@ class RestBackend(AbstractBackend):
                 raise Exception("You cannot pass different uris to the same "
                                 "backend")
 
-        # custom redirecthandler for testing purposes
-        #class CustomRedirectHandler(urllib2.HTTPRedirectHandler):
-        #    def redirect_request(self, req, fp, code, msg, headers, newurl):
-        #        print code; print msg; print headers; print newurl
-        #        return urllib2.HTTPRedirectHandler.redirect_request(self,
-        # req, fp, code, msg, headers, newurl)
-        #opener = urllib2.build_opener(CustomRedirectHandler())
-        #class SmartRedirectHandler(urllib2.HTTPRedirectHandler):
-        #    def http_error_302(self, req, fp, code, msg, headers):
-        #        result = urllib2.HTTPRedirectHandler.http_error_302(self, req,
-        #                                                fp, code, msg, headers)
-        #        result.status = code
-        #        result.headers = headers
-        #        return result
-
-        if debug:
-            handler=urllib2.HTTPHandler(debuglevel=1)
-            opener = urllib2.build_opener(handler)
+        if httphandler:
+            if isinstance(httphandler, list):
+                opener = urllib2.build_opener(*httphandler)
+            else:
+                opener = urllib2.build_opener(httphandler)
         else:
             opener = urllib2.build_opener()
 
         if extra_headers:
             self.GET_headers.update(extra_headers)
 
+        reference_time = datetime.datetime.now()
+
         request = urllib2.Request(url=uri, headers=self.GET_headers)
 
         result_file = opener.open(request)
 
+        now = datetime.datetime.now()
+        self.lookup_time = now - reference_time
+
         if result_file.geturl() != uri:
             logger.info("%s was redirected. Content url: %r" % (uri,
                                                      result_file.geturl()))
@@ -119,10 +112,6 @@ class RestBackend(AbstractBackend):
             logger.info("Content-Length: %s"
                         % result_file.headers["Content-Length"])
 
-        #print result_file.info()
-        #print "os.stat().st_size returns:", os.stat("out.txt").st_size
-        #f = result_file; f.seek(0,2); print f.tell()  #os.fstat(f.fileno())[6]
-
         if not "Content-Type" in result_file.headers:
             raise FiletypeMappingError("No Content-Type specified in response")
         self.content_type = result_file.headers['Content-Type'].split(";")[0]
@@ -130,8 +119,9 @@ class RestBackend(AbstractBackend):
         # Many servers don't do content negotiation: if one of the following
         # content_types are returned by server, assume the mapped type
         overwrite_content_type_map = {
-            "text/plain": "application/rdf+xml"
+            "text/plain": "application/rdf+xml",
         }
+
         if self.content_type in overwrite_content_type_map:
             self.content_type = overwrite_content_type_map[self.content_type]
 
@@ -145,10 +135,11 @@ class RestBackend(AbstractBackend):
                                    % self.content_type)
 
         format = file_extension.strip(".")
-        if format in ["rdf", "ksh"]:
-            # fix responses that we know are wrong
-            format = "xml"
 
+        # assure format is correct
+        if format in ["rdf", "ksh"]:
+            format = "xml"
+        # check if rdflib parser exists for format
         assure_parser_plugin_exists(format)
 
         self.format = format
@@ -163,28 +154,20 @@ class RestBackend(AbstractBackend):
             "Content-Length": str(len(data)),
         })
 
-        # TODO: authentication? oauth?
-        #h = httplib2.Http()
-        #h.add_credentials('name', 'password')
-        #resp, content = h.request(uri, "PUT", body=data, headers=headers)
-        #if resp.status != 200: raise Error(resp.status, errmsg, headers)
-        #return resp, content
-
         opener = urllib2.build_opener(urllib2.HTTPHandler)
         request = urllib2.Request(self.uri,
                                   data=data,
                                   headers=self.PUT_headers)
         request.get_method = lambda: 'PUT'
         response = opener.open(request)
-        # assert status_code 200, 201?
 
 
 class FileBackend(AbstractBackend):
-    """Manages one xml file --> Uri that the user wants to "PUT" to is not
-    flexible!
-    """
+    """Manages one xml file as a data basis"""
 
-    def __init__(self, filename, format=None):
+    def __init__(self, filename,
+                 format=None,
+                 store_old_versions=True):
         assert os.path.exists(filename)
 
         format = format if format else guess_format_from_filename(filename)
@@ -192,8 +175,15 @@ class FileBackend(AbstractBackend):
 
         self.format = format
         self.filename = filename
+        self.store_old_versions = store_old_versions
 
-    def GET(self, uri):
+    def GET(self,
+            uri,
+            extra_headers=None,
+            httphandler=None,
+            ):
+        assert not extra_headers, "Not Implemented"
+        assert not httphandler, "Not Implemented"
         if not hasattr(self, "uri"):
             self.uri = uri
         else:
@@ -206,9 +196,9 @@ class FileBackend(AbstractBackend):
         return data
 
     def PUT(self, data):
-        assert self.uri, "GET has to be called before PUT possible"
+        assert self.uri, "GET has to be called before PUT"
 
-        if os.path.exists(self.filename):
+        if os.path.exists(self.filename) and self.store_old_versions:
             # File already exists. Make backup copy
             now = datetime.datetime.strftime(datetime.datetime.utcnow(),
                                              '%Y%m%d-%H%M%S')
@@ -219,13 +209,15 @@ class FileBackend(AbstractBackend):
                     now, file_extension)
             else:
                 old_version = u"%s_%s" % (self.filename, now)
+            self.old_version = old_version
             shutil.copy(self.filename, old_version)
 
         with open(self.filename, "w") as f:
             f.write(data)
-        self.old_version = old_version
 
     def revert_to_old_version(self):
+        assert self.store_old_versions, ("This FileBackend is not configured "
+            "to store old versions")
         if hasattr(self, "old_version"):
             logger.info("Reverting to version before last saved version")
             shutil.copy(self.old_version, self.filename)
@@ -239,7 +231,13 @@ class MemoryBackend(AbstractBackend):
         assure_parser_plugin_exists(format)
         self.format = format
 
-    def GET(self, uri):
+    def GET(self,
+            uri,
+            extra_headers=None,
+            httphandler=None,
+            ):
+        assert not extra_headers, "Not Implemented"
+        assert not httphandler, "Not Implemented"
         return self.data
 
     def PUT(self, data):
